"""Define project metadata
"""

__all__ = [
    '__title__',
    '__summary__',
    '__url__',
    '__version__',
    '__author__',
    '__email__',
    '__license__',
]

__title__ = 'carafe'
__summary__ = 'Collection of Flask extensions geared towards JSON APIs'
__url__ = 'https://github.com/dgilland/carafe'

__version__ = '0.4.0'

__author__ = 'Derrick Gilland'
__email__ = 'dgilland@gmail.com'

__license__ = 'MIT License'
