"""Collection of Flask extensions geared towards JSON APIs.
"""

from .app import FlaskCarafe
from .client import Client, JSONClient

from .__meta__ import (
    __title__,
    __summary__,
    __url__,
    __version__,
    __author__,
    __email__,
    __license__
)
